#!/usr/bin/python

from sys     import argv
from os.path import join
from math    import floor, ceil, log10

#-----------------------------------------------------------------------

def header (out, prologues):
    out.write('input linechart.mp;\n\n')
    out.write('prologues := %d;\n' % prologues)
    out.write('u  :=  1mm;\n')
    out.write('wd := 60mm;\n')
    out.write('ht := 36mm;\n')
    out.write('\n')
    out.write('color gray;   gray   = 0.7white;\n')
    out.write('color lred;   lred   = 0.5red  +0.5white;\n')
    out.write('color lblue;  lblue  = 0.4blue +0.6white;\n')
    out.write('color lgreen; lgreen = 0.5green+0.5white;\n')
    out.write('color dgreen; dgreen = 0.7green;\n')
    out.write('color yellow; yellow = (1,0.7,0);\n')
    out.write('color purple; purple = (0.7,0,0.9);\n')
    out.write('\n')

#-----------------------------------------------------------------------

def footer (out):
    out.write('end.\n')

#-----------------------------------------------------------------------

def read (fname):
    data = []
    with open(fname, 'r') as inp:
        for line in inp:
            line = line.split()
            if line[0] == 'test': continue
            supp = int(line[1])
            sets = int(line[2])
            time = float(line[3])
            data.append((line[0], supp, sets, time))
    return data

#-----------------------------------------------------------------------

def sets (data, test):
    return [(s,log10(n)) for x,s,n,t in data if x == test]

#-----------------------------------------------------------------------

def time (data, test):
    return [(s,log10(t)) for x,s,n,t in data if x == test]

#-----------------------------------------------------------------------

def diag (out, fid, pfx, name, curves):
    x = [x for p,c,s,t in curves for x,y in p]
    xmin,xmax = min(x),max(x); xrng = xmax-xmin
    if xmax > 1000:      xmax += 0.04*xrng
    else:                xmax += 0.03*xrng
    if xmin < 0.06*xrng: xmin  = 0
    else:                xmin -= 0.03*xrng
    xrng = xmax-xmin
    y = [y for p,c,s,t in curves for x,y in p]
    ymin,ymax = min(y),max(y); yrng = ymax-ymin
    if ymin < -1.1: ymin  = -1.1
    else:           ymin -= 0.03*yrng
    if ymax >  9.9: ymax  =  9.9
    else:           ymax += 0.03*yrng
    yrng = ymax-ymin
    out.write('beginfig(%d); %% %s\n' % (fid,pfx+'_'+name))
    out.write('  pair data[];\n')
    out.write('  linechart.set_defaults("ptmr8r");\n')
    out.write('  linechart.set_units (wd/%.5f,'        % xrng)
    out.write(                      ' ht/%.5f);\n'     % yrng)
    out.write('  linechart.set_ranges((%.5f,%.5f), '   % (xmin,xmax))
    out.write(                       '(%.5f,%.5f));\n' % (ymin,ymax))
    if   xrng <=  16: dx =   1
    elif xrng <=  80: dx =   5
    elif xrng <= 160: dx =  10
    elif xrng <= 600: dx =  50
    elif xrng <= 999: dx = 100
    else:             dx = 200
    xs = int(ceil (xmin/dx))*dx
    xe = int(floor(xmax/dx))*dx
    out.write('  linechart.draw_xticks(%d' % xs)
    for x in range(xs+dx,xe+1,dx): out.write(', %d' % x)
    out.write(');\n')
    out.write('  linechart.draw_yticks(%d' % int(ceil(ymin)))
    for y in range(int(ceil(ymin))+1, int(floor(ymax))+1):
        out.write(', %d' % y)
    out.write(');\n')
    y = int(floor(ymin))
    out.write('  linechart.draw_ylogs(%d, %d);\n'
              % (y,int(ceil(ymax))-y+1))
    out.write('  linechart.draw_frame;\n')
    for i in range(len(curves)):
        c = curves[i]           # traverse the curves
        if len(c[0]) <= 0 and c[1][0:3] != 'w/o': continue
        out.write('  linechart.pfill(data')
        for p in c[0]: out.write(', (%d,%.5f)' % p)
        if len(c[0]) <= 0:
            out.write(', (%d,%.5f)' % (xmax-0.1*xrng,ymax-0.1*yrng))
        out.write(');\n')
        out.write('  linechart.draw_curve(data, "%s",' % c[1])
        out.write(' %d, 0,' % (-1 if c[1] == '' else -2))
        out.write(' %s, %s);\n' % (c[2],c[3]))
    out.write('  linechart.draw_legend(-0.5mm*(1,1), "%s");\n' % name)
    out.write('endfig;\n\n')

#-----------------------------------------------------------------------

if __name__ == '__main__':
    fid = 0                     # initialize the figure index
    with open(argv[1] if len(argv) > 1 else 'bench.mp', 'w') as out:
        header(out, 2)          # write the file header
        for dataset in ['census', 'chess', 'mushroom', 'retail',
                        'T10I4D100K', 'webview1']:
            data   = read(join('..','..','data',dataset+'.res'))
            curves = [[sets(data,'eclat'),     'frequent','black', ''],
                      [sets(data,'eclat-c'),   'closed',  'blue',  ''],
                      [sets(data,'eclat-m'),   'maximal', 'red',   '']]
            diag(out, fid, 'fis', dataset, curves); fid += 1

            curves = [[time(data,'apriori'),   'Apriori', 'black', ''],
                      [time(data,'eclat'),     'Eclat',   'blue',  ''],
                      [time(data,'lcm'),       'LCM',     'red',   ''],
                      [time(data,'fpgrowth'),  'FPgrowth','dgreen',''],
                      [time(data,'sam'),       'SaM',     'yellow',''],
                      [time(data,'relim'),     'RElim',   'purple','']]
            diag(out, fid, 'all', dataset, curves); fid += 1

            curves = [[time(data,'apriori-c'), 'Apriori', 'black', ''],
                      [time(data,'eclat-c'),   'Eclat',   'blue',  ''],
                      [time(data,'lcm-c'),     'LCM',     'red',   ''],
                      [time(data,'fpgrowth-c'),'FPgrowth','dgreen','']]
            diag(out, fid, 'clo', dataset, curves); fid += 1

            curves = [[time(data,'apriori-m'), 'Apriori', 'black', ''],
                      [time(data,'eclat-m'),   'Eclat',   'blue',  ''],
                      [time(data,'lcm-m'),     'LCM',     'red',   ''],
                      [time(data,'fpgrowth-m'),'FPgrowth','dgreen','']]
            diag(out, fid, 'max',  dataset, curves); fid += 1

            curves = [[time(data,'apriori'),   'Apriori', 'black', ''],
                      [time(data,'eclat'),     'Eclat',   'blue',  ''],
                      [time(data,'lcm'),       'LCM',     'red',   ''],
                      [time(data,'fpgrowth'),  'FPgrowth','dgreen',''],
                      [time(data,'apriori-l'), 'w/o m16', 'gray',  ''],
                      [time(data,'eclat-l'),   '',        'lblue', ''],
                      [time(data,'lcm-l'),     '',        'lred',  ''],
                      [time(data,'fpgrowth-l'),'',        'lgreen','']]
            diag(out, fid, 'm16',  dataset, curves); fid += 1

            curves = [[time(data,'apriori'),   'Apriori', 'black', ''],
                      [time(data,'eclat'),     'Eclat',   'blue',  ''],
                      [time(data,'lcm'),       'LCM',     'red',   ''],
                      [time(data,'fpgrowth'),  'FPgrowth','dgreen',''],
                      [time(data,'apriori-x'), 'w/o pex', 'gray',  ''],
                      [time(data,'eclat-x'),   '',        'lblue', ''],
                      [time(data,'lcm-x'),     '',        'lred',  ''],
                      [time(data,'fpgrowth-x'),'',        'lgreen','']]
            diag(out, fid, 'pex',  dataset, curves); fid += 1

            curves = [[time(data,'apriori'),   'Apriori', 'black', ''],
                      [time(data,'eclat-l'),   'Eclat',   'blue',  ''],
                      [time(data,'lcm-l'),     'LCM',     'red',   ''],
                      [time(data,'fpgrowth-l'),'FPgrowth','dgreen',''],
                      [time(data,'apriori-x'), 'w/o pex', 'gray',  ''],
                      [time(data,'eclat-xl'),  '',        'lblue', ''],
                      [time(data,'lcm-xl'),    '',        'lred',  ''],
                      [time(data,'fpgrowth-xl'), '',      'lgreen','']]
            diag(out, fid, 'pxm',  dataset, curves); fid += 1
        footer(out)             # write the file footer
